/*
 * Project name:
      Gyro click
 * Copyright:
      (c) mikroElektronika, 2012.
 * Revision History:
      20130115:
      - Initial release (DO);
 * Description:
      This is a demonstration how Gyro click can be used to measure angular displacement.
      It'is calculated by integrating of angular rate of X Y and Z axis,
      which is measured by Gyroscope module.
      It uses threshold value and zero-rate level cancellation for reducing readout error.
      Example uses UART communication to dispaly data on PC Terminal application.

 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8.000 MHz Crystal, 32.000 MHz MCU Clock
     Ext. Modules:    Gyro click - ac:Gyro_click
                      http://www.mikroe.com/click/gyro/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
      - Place Gyro click board in the mikroBUS socket 1.
      - Turn ON UART communication SW1.1 and SW2.2
      - Set jumpers J3 and J4 to RS-232 or USB based on desired port.

*/

#include"L3GD20_Driver.h"
#define TSHLD 50

sbit GYRO_CS      at LATE0_bit;  // Chip Select pin
sbit GYRO_CS_Dir  at TRISE0_bit;

sbit DATA_RDY     at PORTE.B1;   // DataReady - DRDY/INT2 pin
sbit DATA_RDY_Dir at TRISE1_bit;

sbit GYRO_INT     at PORTB.B0;   // Interrupt - INT1 pin
sbit GYRO_INT_Dir at TRISB0_bit;

GData value;
long int r0_x, r0_y, r0_z;
float  sumX, sumY, sumZ;
char response;

void MCU_Init(){
  CM1CON = 0;                    // Turn off comparators
  CM2CON = 0;
  ANSELB = 0;                    // Configure pins as digital
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;
  SLRCON = 0;
  
  LATA = 0x00;                   // Reset LAT register values
  LATB = 0x00;
  LATC = 0x00;
  LATD = 0x00;
  LATE = 0x00;

  TRISA = 0x00;                  // Configure pins as output
  TRISC = 0x00;
  TRISC = 0x00;
  TRISD = 0x00;
  TRISE = 0x00;

  UART1_Init(115200);            // Initialize UART
  Delay_ms(100);                 // Wait for UART module to stabilize
  I2C1_Init(100000);             // Initialize I2C
}

void UART_Write_New_Line(){
  UART1_Write_Text("\r\n");
}

void Gyro_Config(){
unsigned short response;
unsigned short i;
unsigned short ID;
char txt[3];

  r0_x = 0;
  r0_y = 0;
  r0_z = 0;
  response = 0;

  response += L3GD20_Configure(L3GD20_CTRL_REG1, 0x1F);   // Turn ON module (normal mode), enable X, Y and Z axis , ODR=95Hz, Cut-Off=25
  response += L3GD20_Configure(L3GD20_CTRL_REG4, 0x00);   // Full scale selection - 250dps
  response += L3GD20_Configure(L3GD20_CTRL_REG3, 0x08);   // Enable Date-ready on DRDY/INT2 pin
  if (response == 3){                                     // Sum of all resposne values should be 3
    UART1_Write_Text("Gyro module initialized.");
    UART1_Write_Text("\r\n");
  }
  else{
    UART1_Write_Text("Error: Gyro Config!");
    UART1_Write_Text("\r\n");
  }
  
  ID = Identify_Gyro();                                   // Get ID from Gyroscope module
  ByteToHex(ID, txt);                                     // Convert byte value to HEX-String
  UART1_Write_Text("GyroID: ");                           //
  UART1_Write_Text(txt);                                  //
  UART_Write_New_Line();                                  //
  UART1_Write_Text("Calibration: Module should be in rest position.");
  UART_Write_New_Line();  
  for (i = 0 ; i < 64 ; i++ ){                            // Collect 64 values in rest position
    L3GD20_GetAngRateRaw(&value);
    r0_x += value.X_AXIS;                                 // Sum the values
    r0_y += value.Y_AXIS;                                 // for every
    r0_z += value.Z_AXIS;                                 // axis
  }
  r0_x >>= 6;                                             // Find avreage reding in resting position
  r0_y >>= 6;                                             // and use them as correction values
  r0_z >>= 6;                                             // for all three axis
  UART1_Write_Text("Calibration: Done!");
  UART_Write_New_Line();
}

void DisplayAngleRate(char *str, float val){
char txt[20];
  UART1_Write_Text("| ");
  UART1_Write_Text(str);
  UART1_Write_Text(": ");
  sprintf(txt, "%7.2f", val);
  UART1_Write_Text(txt);
  UART1_Write_Text(" deg |");
}

void NewLine(){
  UART1_Write_Text("\r\n");
}

void main() {
  int Xval, Yval, Zval;
  char display_flag = 0;

  MCU_Init();
  Gyro_Init();
  Gyro_Config();

  NewLine();
  NewLine();
  
  sumX = 0;
  sumY = 0;
  sumZ = 0;
  display_flag = 0;
  
  while(1){
    response = L3GD20_GetAngRateRaw(&value);              // Colect new set of values
    if (response == 1){
      if (abs(value.X_AXIS - r0_x) > TSHLD){              // If corrected value of X axis is over treshold level use the vale for integration
         Xval = value.X_AXIS - r0_x;                      // Calculate Angular rate for X axis
         sumX += Xval * 0.00875 / 94.8 * 3.0;             // Multuply with dps factor for 250dps i's 8.75 mdps/sec (0,00875)
                                                          //   frequecy of readig values from module ~95Hz(94.8) - three chanel (3.0)
         display_flag = 1;                                // Raise the flag for data to be displayed via UART
      }
      if (abs(value.Y_AXIS - r0_y) > TSHLD){              // If corrected value of Y axis is over treshold level use the vale for integration
        Yval = value.Y_AXIS - r0_y;                       // Calculate Angular rate for Y axis
        sumY += Yval  * 0.00875 / 94.8 * 3.0;             // Multuply with dps factor for 250dps i's 8.75 mdps/sec (0,00875)
                                                          //   frequecy of readig values from module ~95Hz(94.8) - three chanel (3.0)
        display_flag = 1;                                 // Raise the flag for data to be displayed via UART
      }
      if (abs(value.Z_AXIS - r0_z) > TSHLD){              // If corrected value of Z axis is over treshold level use the vale for integration
         Zval = value.Z_AXIS - r0_z;                      // Calculate Angular rate for Z axis
         sumZ += Zval * 0.00875 / 94.8 * 3.0;             // Multuply with dps factor for 250dps i's 8.75 mdps/sec (0,00875)
                                                          //   frequecy of readig values from module ~95Hz(94.8) - three chanel (3.0)
         display_flag = 1;                                // Raise the flag for data to be displayed via UART
      }
      if(display_flag){                                   // If change was detected, send data via UART
        DisplayAngleRate("sX", sumX);
        DisplayAngleRate("sY", sumY);
        DisplayAngleRate("sZ", sumZ);
        NewLine();
        display_flag = 0;                                 // Reset flag variable
      }
    }
    else
    {
      UART1_Write_Text("Error reading data!");
    }
  }
}